library ieee;
use ieee.numeric_std.all;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;

entity TxReset is
port (
	clk             : in  std_logic;
	POWER_UP_RST_N  : in  std_logic; 						-- similar to reset signal
	pll_locked      : in  std_logic_vector (0 downto 0);
	pll_areset      : out std_logic_vector (0 downto 0);    -- similar to pll_powerdown
	tx_digitalreset : out std_logic_vector (0 downto 0);
	tx_ready        : out std_logic; -- for rx reset machine
	debug_st        : out std_logic_vector (1 downto 0) -- will be synthesized away if not used
);
end TxReset;

architecture structure of TxReset is
	signal POWER_UP_RST_N_stable : std_logic;
	type state_type is (STATE_RESET, STATE_PLLREADY, STATE_TXREADY);
	signal pres_state, next_state : state_type;
	attribute syn_encoding : string;
	attribute syn_encoding of state_type : type is "safe";
	-- synchronization registers
	signal pll_locked_d : std_logic_vector(2 downto 0);
	signal pll_locked_stable : std_logic;
begin


-- debug_st => open,

-- FSM sequential logic
dff_res: process (clk)
begin
	if clk'event and clk = '1' then  -- rising clock edge
		pll_areset(0) <= not POWER_UP_RST_N;
	end if;
end process dff_res;

-- FSM sequential logic
reset_fsm : process (clk, POWER_UP_RST_N)
begin
	if POWER_UP_RST_N = '0' then  -- asynchronous reset (active low)
		pres_state <= STATE_RESET;
		pll_locked_d <= "000";
		pll_locked_stable <= '0';
	elsif clk'event and clk = '1' then  -- rising clock edge
		pres_state <= next_state;
		pll_locked_d(0) <= pll_locked(0);
		pll_locked_d(1) <= pll_locked_d(0);
		pll_locked_d(2) <= pll_locked_d(1);
		pll_locked_stable <= pll_locked_d(2) and pll_locked_d(1);
	end if;
end process reset_fsm;

-- FSM combinational logic
fsm_comb : process (pres_state, pll_locked_stable)
begin
	next_state <= pres_state;
	case pres_state is
		when STATE_RESET =>
			if pll_locked_stable = '1' then
				next_state <= STATE_PLLREADY;
			end if;
		when STATE_PLLREADY =>
			next_state <= STATE_TXREADY;
		when STATE_TXREADY =>
			if pll_locked_stable = '0' then
				next_state <= STATE_RESET;
			end if;
		when others =>
			next_state <= STATE_RESET;
	end case;
end process fsm_comb;

-- FSM output multiplexer [with an extra register]
out_mux : process (pres_state, clk, POWER_UP_RST_N)
begin
	if POWER_UP_RST_N = '0' then  -- asynchronous reset (active low)
			tx_ready <= '0';
			tx_digitalreset  <= (others => '1');
			debug_st <= "00";
	elsif clk'event and clk = '1' then  -- rising clock edge
		case pres_state is	
			when STATE_RESET =>
				tx_ready <= '0';
				tx_digitalreset  <= (others => '1');
				debug_st <= "00";
			when STATE_PLLREADY =>
				tx_ready <= '0';
				tx_digitalreset  <= (others => '1');
				debug_st <= "01";
			when STATE_TXREADY =>
				tx_ready <= '1';
				tx_digitalreset <= (others => '0');
				debug_st <= "10";
			when others => null;
		end case;
	end if;
end process out_mux;

end structure;
